#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001 Free Software Foundation
#
# FILE:
# BaseMask.py
#
# DESCRIPTION:
#
# NOTES:
#

from gnue.common import GConfig, GDebug
from FormatExceptions import *


# This is a class for a mask literal element
class Literal:
  def __init__(self, literal): 
    self.literal = literal

  def getFormattedValue(self, value): 
    return self.literal

  def isValidEntry(self, value): 
    return value == self.literal

class BaseMask:
  value = ""
  maskMappings = {}
  inputHandlers = []
  outputHandlers = []
  output2Handlers = []
  predefinedLiterals = ""
  basetype = "base"
  defaultmask = ""
  literalClass = Literal

  def __init__(self, outputMask, inputMask=None, outputMask2=None):
    self.outputHandlers  = self.parseMask(outputMask)
    if inputMask != None:
      self.inputHandlers = self.parseMask(inputMask)
    if outputMask2 != None:
      self.output2Handlers = self.parseMask(outputMask2)


  # Take a mask string and break it into its elements and map it to handlers
  def parseMask(self, mask):
    GDebug.printMesg(15,'parseMask("%s")' % mask)
    maskHandler = []

    # Load a predefined mask, if that's what user specified
    # TODO: Should predefined masks be pulled from other than gnue.conf?
    if len(mask) > 1 and mask[0] == '&':
      if mask[1:] == self.basetype:
        mask = GConfig.get('%smask_%s' % (self.basetype, self.basetype), \
          self.defaultmask)
      else:
        try:
          mask = GConfig.get('%smask_%s' % (self.basetype, self.basetype))
        except KeyError:
          raise PredefinedMaskNotFound, \
              'The requested format mask %s is not defined' % mask[1:]

    # Process each character in mask at a time
    isLiteral = 0
    for ch in mask:
      if ch == '\\':
        isLiteral = 1
      elif isLiteral or ch in self.predefinedLiterals:
        isLiteral = 0
        maskHandler.append(self.literalClass(ch))
      elif self.maskMappings.has_key(ch):
        maskHandler.append(self.maskMappings[ch]())
      else:
        raise InvalidCharInMask, 'Unexpected character "%s" in %s mask.' \
            % (ch, self.basetype)

    return maskHandler

  # If secondary is true, then use secondary mask (outputMask2)
  def getFormattedOutput(self, secondary=0):
    pass

  def getFormattedInput(self):
    pass

  def processEdit (self, str, pos=0, replaces=0):
    pass

# This is a base class for mask elements
class MaskSection:
  def __init__(self):
    self.minLength = 0
    self.maxLength = 0

    self.intersperse = 0
    self.intersperseString = ","
    self.intersperseRepeat = 1


  def getFormattedValue(self):
    return self.value

  def isValidEntry(self, value):
    return 0




class MaskInputInstance: 
  def __init__(self, handler):
    self.handler = handler
    self.index = 0
    self.cursor = 0
    self.display = "" 
    self.input = ""
    self.inputMaskPos = []
    self.inputMaskLen = []
    for i in range(len(self.handler.inputHandlers)):
      self.inputMaskPos.append(0)
      self.inputMaskLen.append(0)

  def setText(self, text): 
    self.input = text
    self.display = text
    self.cursor = 0
    self.index = 0

  def isValid(self): 
    return 1

  def addText(self, text, pos, replaces): 
    pass

  def deleteText(self, text, pos, characters): 
    pass

  def cursorToIndex(self, val): 
    return val


  def indexToCursor(self, val): 
    return val


  def moveCursor(self, relative): 

    # Direction will be either 1 or -1.
    direction = relative = 0 or int(relative/abs(relative))

    self.cursor = self.cursor + relative

    if self.cursor <= 0: 
      self.cursor = 0 
      self.direction = 1
    elif self.cursor >= len(self.display):
      self.cursor = len(self.display)
      self.direction = -1

    section = 0
    while section < len(self.handler.inputMaskPos) and \
          self.handler.inputMaskPos[section] < pos:
      section += 1

    if section == len(self.handler.inputMaskPos): 
      self.cursor = len(self.display)

    elif isinstance(self.handler.inputMaskPos[section], Literal):
      self.moveCursor(self, direction)

    self.index = self.indexToCursor(self.cursor)

    return self.cursor


  def processEdit (self, value, str, pos=0, replaces=0):
    GDebug.printMesg(15,'processEdit(%s,%s,%s,%s)' % (value,str,pos,replaces))
    nv = value
    if pos == len(nv):
      nv += str
    else:
      nv = "%s%s%s" % (nv[:pos],str,nv[pos+replaces:])

    GDebug.printMesg(15,"inputMaskPos=%s" % self.inputMaskPos)

    section = 0
    while section < len(self.handler.inputMaskPos) and \
          self.handler.inputMaskPos[section] < pos:
      section += 1
    i = pos
    while i < len(nv):
      if not self.handler.inputHandlers[section]\
           .isValidEntry(nv[self.handler.inputMaskPos[section]:i+1]):
        if i == self.handler.inputMaskPos[section] and \
           not isinstance(self.handler.inputHandlers[section],Literal):
          self.handler.inputMaskLen[section] = i - self.handler.inputMaskPos[section]
          return nv
        else:
          self.handler.inputMaskLen[section] = i - self.handler.inputMaskPos[section]
          if section == len(self.handler.inputHandlers) - 1:
            if i != len(nv)-1:
              return nv
          else:
            section += 1
            self.inputMaskPos[section] = i
            self.inputMaskLen[section] = 0
      else:
        i += 1

    self.handler.inputMaskLen[section] = i - self.handler.inputMaskPos[section]

    for i in range(section+1, len(self.handler.inputHandlers)):
      self.handler.inputMaskLen[i] = 0

    GDebug.printMesg(15,"<< %s" % nv)
    return nv




