#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001 Free Software Foundation
#
# FILE:
# GComm.py
#
# DESCRIPTION:
# Class that provides a basis for GNUe Comm clients & servers.
#
# NOTES:
#

import GDebug

#
# Attach to a client driver
#
def attach(interface, params):
  try:
    driver = dyn_import('gnue.common.commdrivers.%s' % interface)
  except ImportError, mesg:
    GDebug.printMesg(1, \
       "Unable to import GComm Adapter '%s': \n%s" % (driver,mesg))
    raise InvalidAdapter, mesg

  if not (hasattr(driver,'CLIENT') and driver.CLIENT):
    raise NoClientAdapter, \
           "GComm adapter '%s' does not support clients" % interface

  adapter = driver.Client(params)

  return adapter


#
# Export our services
#
def export(driver, schema, params):
  pass


#
# Exception hierarchy:
#
# Error
# |__ ProgrammingError
# |   |__ InvalidService
# |__ CommunicationsError
# |__ UserError (base for user-defined errors)
#

class Error(StandardError):
  pass

class InvalidAdapter(Error):
  pass

class NoClientAdapter(Error):
  pass

class NoServerAdapter(Error):
  pass

class ProgrammingError(Error):
  pass

class InvalidAdapter(ProgrammingError):
  pass

class InvalidService(ProgrammingError):
  pass

class CommunicationsError(Error):
  pass

class UserError(Error):
  pass


