#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public 
# License along with program; see the file COPYING. If not, 
# write to the Free Software Foundation, Inc., 59 Temple Place 
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000, 2001 Free Software Foundation
#
# FILE:
# geas/DBdriver.py
#
# DESCRIPTION:
# Implementation of dbdriver for use with GNUe Application Server.
#
# NOTES:
# This implementation currently requires the use of python-orbit 
# bindings.  You MUST use orbit-python 0.3.0 or newer 
#
# HISTORY:
#

from gnue.common import GDataObjects, GDebug, GConditions, GConfig
import string
import types
import md5
import sys

try:
    import CORBA
    idlFiles=GConfig.get('GEASDIR')+"/share/idl/geas.idl"
    GDebug.printMesg(1,"IDL being loaded from %s" % (idlFiles))
    CORBA._load_idl(idlFiles)
    import GEAS
    
except ImportError,ex:
    print "Failed to load CORBA module."
    print "Please see http://orbit-python.sault.org/ for the required python CORBA bindings"
    sys.exit(0)


class GEAS_RecordSet(GDataObjects.RecordSet): 
  def _postChanges(self): 
    values = []
    fields = []
    fieldString = ""
    for field in (self._modifiedFields.keys()): 
      fields.append(field + " = %s")
      values.append(self._fields[field])
    
    # statement = 'UPDATE %s SET %s WHERE %s = %s'

# GEAS_ResultSet
#
# Notes:
# In the GEAS driver a CURSOR is simply the ObjectList handle returned
# via the query interface
#
class GEAS_ResultSet(GDataObjects.ResultSet): 
  def __init__(self, dataObject, cursor=None, \
        defaultValues={}, masterRecordSet=None): 
    GDataObjects.ResultSet.__init__(
           self,dataObject,cursor,defaultValues,masterRecordSet)
    self._recordSetClass = GEAS_RecordSet
    self._fieldNames = None
    GDebug.printMesg(5, 'ResultSet created')
  
  def _loadNextRecord(self): 
    if self._cursor: 
      try:
        print self._cursor
        
        if not self._fieldNames:
            self._dataObject.getSchemaList()
        #  self._fieldNames = []
        #  for t in (self._cursor.description): 
        #    self._fieldNames.append (t[0])
        #    self._boundFields[string.lower[t[0]]] = ""


        for obj in self._cursor.objects:
            print "Obj ", obj.objectID
            #self.objlist[obj.objectID] = obj
            #r = self._buildMask(obj)
            #print "Mask", r
            #rs.append(r)
            
#        rs = self._cursor.fetchone()
      except self._dataObject._DatabaseError, err:
        raise GDataObjects.ConnectionError, err

#      GDebug.printMesg(5, rs)
#      if rs: 
#        i = 0
#        dict = {}
#        for f in (rs): 
#          dict[self._fieldNames[i]] = f
#          i = i + 1
#        self._cachedRecords.append (self._recordSetClass(parent=self, \
#                                            initialData=dict))
#        return 1
#      else:
#        return 0
#    else: 
#      return 0

#  def _buildMask(self, obj):
#        r = {}
#        # add objectID into field list
#        r["ID"] = obj.objectID
#        for f in self._fieldNames.keys():
#         for f in obj
#            r[f] = self._get_object_field(obj,f)
#        return r
#    
#  def _get_object_field(self,obj,field):
#	retval = ""
#	try:
#	    retval = obj.getField(field)
#	except (GEAS.UnknownField,GEAS.NotReadable,GEAS.ServerError,
#	        GEAS.Interrupted,GEAS.NotDataField,GEAS.Locked,
#		GEAS.TransactionNotInProgress),ex:
#	    # pass error message on
#	    raise DBError , ex.detail
#	except GEAS.NullField,ex:
#	    # allowed, the field defaults to 'empty'
#	    pass
#	return retval


class GEAS_DataObject(GDataObjects.DataObject): 

  conditionElements = {
       'add':             (1, 2, 999, '(%s)',                   '+'      ),
       'sub':             (1, 2, 999, '(%s)',                   '-'      ),
       'mul':             (1, 2, 999, '(%s)',                   '*'      ),
       'div':             (1, 2, 999, '(%s)',                   '/'      ),
       'and':             (1, 2, 999, '(%s)',                   ' AND '  ), 
       'or':              (1, 2, 999, '(%s)',                   ' OR '   ), 
       'not':             (1, 1,   1, '(NOT %s)',               None     ), 
       'negate':          (1, 1,   1, '-%s',                    None     ),
       'eq':              (1, 2,   2, '(%s = %s)',              None     ),
       'ne':              (1, 2,   2, '(%s != %s)',             None     ),
       'gt':              (1, 2,   2, '(%s > %s)',              None     ),
       'ge':              (2,   2, '(%s >= %s)',                None     ),
       'lt':              (2,   2, '(%s < %s)',                 None     ),
       'le':              (2,   2, '(%s <= %s)',                None     ),
       'like':            (2,   2, '%s LIKE %s',                None     ),
       'notlike':         (2,   2, '%s NOT LIKE %s',            None     ),
       'between':         (3,   3, '%s BETWEEN %s AND %s',      None     ) }

  def __init__(self): 
    GDataObjects.DataObject.__init__(self)

    GDebug.printMesg (1,"GEAS database driver backend initializing")
    self._resultSetClass = GEAS_ResultSet
    self._DatabaseError = GEAS.ServerError


  def connect(self, connectData={}): 
    GDebug.printMesg(1,"GEAS database driver connecting...")

    # Extract the username and password...
    user = connectData['_username']
    passwd = connectData['_password']

    # find the object server base object
    self.orb = CORBA.ORB_init(() , CORBA.ORB_ID)
    try:
      print GConfig.get('GEASDIR')+"/var/run/geas-server.ior"
      self.ior = open(GConfig.get('GEASDIR')+"/var/run/geas-server.ior").readline()
    except (IOError), ex:
      GDebug.printMesg(1, "*** Error: ior file for geas was not found...  is geas running?")
      raise GDataObjects.LoginError, 'Unable to connect to GEAS. Is it running?'

    self._factory = self.orb.string_to_object(self.ior)
    try:
      GDebug.printMesg(1,"Logging into geas as %s" % (user))
      if (user == 'guest') or (user == ''):

        # TODO: (jcater) I'm not so sure about guest logins. This
        # TODO: (jcater) should be discussed. At a minimum, I think
        # TODO: (jcater) we need gnue.conf flags: 
        # TODO: (jcater)     allowGuestLogin = [0|1]
        # TODO: (jcater)     guestLogin = guest
        # TODO: (jcater)     allowBlankLogin = [0|1]
        self._dataConnection = self._factory.guestLogin()
      else:

        # TODO: (jcater) The GEAS docs say this is not supported and 
        # TODO: (jcater) CORBA-based security should instead be used. 
        # TODO: (jcater) As I don't know jack about CORBA security, 
        # TODO: (jcater) this will have to do for now (even though, 
        # TODO: (jcater) IIRC, is does nothing to validate the user/pass
        # TODO: (jcater) supplied to it.)
        self._dataConnection = self._factory.unsecureLogin(user,passwd)

      self.triggerExtensions = TriggerExtensions(self._dataConnection)

    except (GEAS.ServerError, GEAS.AuthenticationError), ex:
      raise GDataObjects.ConnectionError, ex

    #
    # Enter transactional mode...
    #   TODO: (jcater) As transaction support does not yet exist in GEAS, 
    #   TODO: (jcater) I have no idea if this works, if it will change, 
    #   TODO: (jcater) or, well, anything for that matter. Think of it  
    #   TODO: (jcater) as a placeholder.
    self._transaction = self._dataConnection.new()


  # We only need the basics -- username and password -- to log in
  def getLoginFields(self): 
    return [['_username', 'User Name',0],['_password', 'Password',1]]

  def _buildQuery(self, conditions={}):
    GDebug.printMesg(7,'Implicit Fields: %s' % self._fieldReferences)
    query = self._dataConnection.newQuery()
    query.classname = "%s" % self.table
    query.orderby = "name"
    query.reverse = CORBA.FALSE
    query.logic = GEAS.Query.AND
    
    print "Conditions" , conditions, ' classname ', self.table

    if len(self._fieldReferences):
###      for fieldname in self._fieldReferences.keys():
      for fieldname in conditions.keys():
        f = GEAS.Query.Field( field=fieldname, test=GEAS.Query.equals,invert=CORBA.FALSE,
                              casesensitive=CORBA.FALSE,value=str(conditions[fieldname]))
        query.addField( f )
        
#        f1 = GEAS.Query.Field( field="name",test=GEAS.Query.equals,
#                               invert=CORBA.FALSE,casesensitive=CORBA.FALSE,value=fieldcontents)
#        
#        f2 = GEAS.Query.Field( field="code",test=GEAS.Query.equals,
#                               invert=CORBA.FALSE,casesensitive=CORBA.FALSE,value=language)
#        query.addField(f2)
#        f3 = GEAS.Query.Field( field="dialect",test=GEAS.Query.equals,
#                               invert=CORBA.FALSE,casesensitive=CORBA.FALSE,value=dialect)
#        query.addField(f3)
#        obj_list = con.executeQuery(query)
#        query.release()
#      
#            self._conditionToSQL(conditions))
#    else: 
#      q = "SELECT * FROM %s%s" % (self.table, self._conditionToSQL(conditions))
#
#    GDebug.printMesg(5,q)
#
    return query


  def _createEmptyResultSet(self, readOnly=0, masterRecordSet=None):
    return self.createResultSet(readOnly=readOnly, masterRecordSet=masterRecordSet)

  def _createResultSet(self, conditions={}, readOnly=0, masterRecordSet=None): 
    try:
      if len(conditions):
        obj_list = self._dataConnection.executeQuery(self._buildQuery(conditions))  
      else:
        obj_list = self._dataConnection.loadAll(self.table)
      
    except self._DatabaseError, err: 
      raise GDataObjects.ConnectionError, err
    rs = self._resultSetClass(self, cursor=obj_list, masterRecordSet=None)
    if readOnly: 
      rs._readonly = readOnly
    return rs

  def commit(self): 
    GDebug.printMesg (5,"GEAS database driver: commit()")
 
    try: 
      self._dataConnection.commit()
    except self._DatabaseError, value:
      raise GDataObjects.ConnectionError, value

  def rollback(self): 
    GDebug.printMesg (5,"GEAS database driver: rollback()")

    try: 
      self._dataConnection.abort()
    except: 
      pass	# I'm SURE this isn't right (jcater)
                # But not all db's support transactions
                # TODO : (jamest) This is cruft from _dbsig driver
                # TODO : not sure what to make of it yet for geas
                
  # Used to convert a condition tree to an sql where clause
  def _conditionToSQL (self, condition): 
    if condition == {} or condition == None: 
      return ""
    elif type(condition) == types.DictType: 
      cond = GConditions.buildConditionFromDict(condition)
    else: 
      cond = condition
  
    if not len(cond._children): 
      return ""
    elif len(cond._children) > 1: 
      chillun = cond._children
      cond._children = []
      _and = GConditions.GCand(cond)
      _and._children = chillun
  
    return " WHERE (%s)" % (self.__conditionToSQL (cond._children[0]))
  
  # Used internally by _conditionToSQL
  def __conditionToSQL (self, element): 
    if type(element) != types.InstanceType: 
      return "%s" % element
    else: 
      otype = string.lower(element.getObjectType()[2:])
      if otype == 'field': 
        return "%s" % element.name
      elif otype == 'const': 
        return "'%s'" % element.value
      elif otype == 'param': 
        return "'%s'" % element.getValue()
      elif self.conditionElements.has_key(otype): 
        for i in range(0, len(element._children)): 
          element._children[i] = self.__conditionToSQL(element._children[i])
        if len(element._children) < self.conditionElements[otype][0]: 
          raise GConditions.ConditionError, \
            'Condition element "%s" expects at least %s arguments; found %s' % \
                (otype, self.conditionElements[otype][0], len(element._children))
        if len(element._children[i]) < self.conditionElements[otype][1]: 
          raise GConditions.ConditionError, \
            'Condition element "%s" expects at most %s arguments; found %s' % \
                (otype, self.conditionElements[otype][0], len(element._children))
        if self.conditionElements[otype][4] == None: 
          return self.conditionElements[otype][3] % tuple(element._children)
        else: 
          return self.conditionElements[otype][3] % \
            (string.join(element._children, self.conditionElements[otype][4]))
      else: 
        raise GConditions.ConditionNotSupported, \
          'Condition clause "%s" is not supported by this db driver.' % otype


  # Return a list of the types of Schema objects this driver provides
  # Doesn't seem to be in use anywhere yet but I'll put it in - jamest
  def getSchemaTypes(self):
      return [('object','Object',1)]

  # Return a list of Schema objects
  def getSchemaList(self, type=None):
      includeObjects = (type in ('object','sources', None))
      
      list = []
      for classname in self._dataConnection.classes:
        list.append(self.getSchemaByName(classname))
      return list

  def getSchemaByName(self, name, type=None): 
      classdef = self._dataConnection.getFullClassDefinition(str(name))
      schema = GDataObjects.Schema(attrs={'id':string.lower(classdef.name),
                                          'name':classdef.name,
                                          'type':'object'},
                                   getChildSchema=self.__getFieldSchema)
      return schema
  
  # Get fields for an object
  def __getFieldSchema(self, parent):
      list = []
      c = con.getFullClassDefinition( classnames[idx-1] )
      #show_classname( c )
      #show_fields( c )
      #raw_input( "Press 'enter' to continue" )


supportedDataObjects = { 
  'object': GEAS_DataObject
}






##############################################################
#
# This is old stuff that needs to go away
#
##############################################################

class DBdriver:
    def __init__ (self):
        GDebug.printMesg(1,"GNU Enterprise Application Server driver initializing")
	self.database = None
        self.orb = None
        self.ior = None
        self.factory = None
        self.uniqueKey = "objectID"
        self.fieldlist = []
        self.objlist = {}

    def disconnect(self):
        print "disconnect"
        self.database.logout();


    def query(self, table, mask = None, order_by=None):
        GDebug.printMesg(3, "query mask %s " % mask)
        (objectID, mask) = self.filterMask(mask)
        self.objlist = {}
        rs = []

	try:
	    if mask or objectID:
		maskfield = None
		maskdata = None
		for k in mask.keys():
		    if mask[k]:
			maskfield = k
			maskdata = mask[k]


		if not maskfield and objectID:
		    maskfield = "objectID"
		    maskdata = str(objectID)

		if maskfield:
		    self.data = self.database.loadObjects(table, maskfield, maskdata)
		else:
		    self.data = self.database.loadAll(table);
	    else:
		self.data = self.database.loadAll(table);

	    if self.data:
		for obj in self.data.objects:
		    self.objlist[obj.objectID] = obj
		    r = self.buildMask(obj)
		    rs.append(r)

	# Handle any errors from the search functions above
	except GEAS.UnknownClass,ex:
	    self.data = NULL
	    print ex.detail
	except GEAS.UnknownField,ex:
	    self.data = NULL
	    print ex.detail

        return rs


    def set_object_field(self,obj,field,value):
	try:
	    if field != "objectid":
		obj.setField(field,value)
	except (GEAS.UnknownField,GEAS.NotWriteable,
	        GEAS.ServerError,GEAS.Interrupted,GEAS.NotDataField,
		GEAS.Locked,GEAS.TransactionNotInProgress,GEAS.Format),ex:
	    raise DBError , ex.detail

    def insert(self,table,mask):
        (objectID, mask) = self.filterMask(mask)

	try:
	    obj = self.database.newObject(table)
	except GEAS.UnknownClass,ex: print ex.detail
        for m in mask.keys():
	    self.set_object_field(obj,m,mask[m])

        mask["objectID"] = obj.objectID

        self.objlist[obj.objectID] = obj
        
        return mask
        
    def delete(self,table,mask):
        (objectID, mask) = self.filterMask(mask)
        obj = self.objlist[objectID]
        if obj:
            obj.delete()

    def update(self,table,mask):
        (objectID, mask) = self.filterMask(mask)
        obj = self.objlist[objectID]
        if obj:
            for k in mask.keys():
		self.set_object_field(obj,k,mask[k])

        return [self.buildMask(obj)]

    def getFieldList(self,table):
        list = {}
        self.fieldlist = {}
        cd = self.database.getFullClassDefinition(table)
        if cd:
            for field in cd.fields:
                if field.type == GEAS.basic:
                    list[field.name] = field.type
	    list["objectid"] = GEAS.Char

        self.fieldlist = list
        return list

    def getUniqueKey(self, table):
        return self.uniqueKey

    def filterMask(self, mask):
        objectID = None
        nmask = {}
        if mask != None:
            for k in mask.keys():
                if k == 'objectID':
                    objectID = mask[k]
                else:
                    nmask[k] = mask[k]

        return (objectID, nmask)

    def _make_passkey(self, user, passwd, random):
        return md5.new(user + '-' + passwd + '-' + random).hexdigest()



#
#  Extensions to Trigger Namespaces
#  
class TriggerExtensions: 

  def __init__(self, connection): 
    self.__connection = connection

  # Return the current date, according to database
#  def getDate(self): 
#    pass

  # Return a sequence number from sequence 'name' 
#  def getSequence(self, name): 
#    pass

  # Run the SQL statement 'statement'
#  def sql(self, statement): 
#    pass
