#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it 
# and/or modify it under the terms of the GNU General Public 
# License as published by the Free Software Foundation; either 
# version 2, or(at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied 
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000, 2001 Free Software Foundation
#
# FILE:
# postgresql/DBdriver.py
#
# DESCRIPTION:
# Postgresql implementation of dbdriver using Python DB-SIG v2
# specification.
#
# NOTES:
#


from string import lower, join
import sys
from gnue.common import GDebug, GDataObjects, GConnections
from gnue.common.dbdrivers._dbsig.DBdriver \
   import DBSIG_RecordSet, DBSIG_ResultSet, DBSIG_DataObject, \
          DBSIG_DataObject_SQL, DBSIG_DataObject_Object

try:
  import pg
  import pgdb as SIG2api
  from _pg import error as PGError   # This is so we can catch login errors
except ImportError, message:
  raise GConnections.AdapterNotInstalled, \
         "Driver not installed: pygresql for PostgreSQL [%s]" % message




class PG_RecordSet(DBSIG_RecordSet):
  pass

class PG_ResultSet(DBSIG_ResultSet):
  def __init__(self, dataObject, cursor=None, defaultValues={}, masterRecordSet=None):
    DBSIG_ResultSet.__init__(self, dataObject, \
            cursor, defaultValues, masterRecordSet)
    self._recordSetClass = PG_RecordSet


  #### NOTE:  Once PyGreSQL's .fetchone() bug is fixed, this method
  ####   should be deleted!  Also note the line in PG_DataObject.__init__.
  ####   Right now, we are having to preload *ALL* rows from a datasource.
  def _loadNextRecord(self):
    GDebug.printMesg(5,'_loadNextRecord() for %s' % self._dataObject.table)
    if self._cursor:
      try:
        rsall = self._cursor.fetchall()
      except self._dataObject._DatabaseError, err:
        raise GDataObjects.ConnectionError, err

      if len(rsall):
        for rs in(rsall):
          GDebug.printMesg(5, "New row retrieved: %s" % rs)
          if not self._fieldNames:
            self._fieldNames = []
            for t in(self._cursor.description):
              self._fieldNames.append(t[0])
            GDebug.printMesg(5, "Field names set to %s" % self._fieldNames)
          if rs: 
            i = 0
            dict = {}
            for f in(rs):
              # TODO - There has got to be a better way
              if type(f) == type(1L):
                f = "%g" % f
              dict[self._fieldNames[i]] = f
              i = i + 1
            self._cachedRecords.append(self._recordSetClass(parent=self,initialData=dict))
          else:
            return 0
        return 1
      else: 
        return 0
    else:
      return 0
####


class PG_DataObject(DBSIG_DataObject):
  def __init__(self):
    DBSIG_DataObject.__init__(self)
    self._DatabaseError = SIG2api.Error
    self._resultSetClass = PG_ResultSet


  def connect(self, connectData={}):
    GDebug.printMesg(1,"Postgresql database driver initializing")
    try:
      self._dataConnection = SIG2api.connect(user=connectData['_username'],
                   password=connectData['_password'],
                   host=connectData['host'],
                   database=connectData['dbname'])
      self.triggerExtensions = TriggerExtensions(self._dataConnection)
    except PGError, value:
      raise GDataObjects.LoginError, value
    except self._DatabaseError, value:
      raise GDataObjects.LoginError, value

    try: 
      encoding = connectData['encoding']
      GDebug.printMesg(1,'Setting postgresql client_encoding to %s' % encoding)
      cursor = self._dataConnection.cursor()
      cursor.execute("SET CLIENT_ENCODING TO '%s'" % encoding)
      cursor.close()
    except KeyError: 
      pass
    except self._DatabaseError: 
      try: 
        cursor.close()
      except: 
        pass

    self._postConnect()


  def _postConnect(self):
    self.triggerExtensions = TriggerExtensions(self._dataConnection)


  #
  # Schema (metadata) functions
  #

  # Return a list of the types of Schema objects this driver provides
  def getSchemaTypes(self):
    return [('view','View',1), ('table','Table',1)]

  # Return a list of Schema objects
  def getSchemaList(self, type=None):
    includeTables = (type in ('table','sources', None))
    includeViews = (type in ('view','sources', None))

    inClause = []
    if includeTables:
      inClause.append ("'r'")
    if includeViews:
      inClause.append ("'v'")

    # TODO: This excludes any system tables and views. Should it?
    statement = "select relname, relkind from pg_class " + \
            "where relkind in (%s) " % (join(inClause,',')) + \
            "and relname not like 'pg_%' " + \
            "order by relname"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():
      list.append(GDataObjects.Schema(attrs={'id':lower(rs[0]), 'name':rs[0],
                         'type':rs[1] == 'v' and 'view' or 'table'},
                         getChildSchema=self.__getFieldSchema))

    cursor.close()
    return list


  # Find a schema object with specified name
  def getSchemaByName(self, name, type=None):
    statement = "select relname, relkind, oid from pg_class " + \
            "where relname = '%s'" % (name)

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    rs = cursor.fetchone()
    schema = GDataObjects.Schema(attrs={'id':rs[2], 'name':rs[0],
                         'type':rs[1] == 'v' and 'view' or 'table'},
                         getChildSchema=self.__getFieldSchema)

    cursor.close()
    return schema

  # Get fields for a table
  def __getFieldSchema(self, parent):

    statement = "select attname, pg_attribute.oid, typname, " + \
            " attnotnull, atthasdef, atttypmod " + \
            "from pg_attribute, pg_type " + \
            "where attrelid = %d and " % (parent.id) + \
            "pg_type.oid = atttypid and attnum >= 0" + \
            "order by attnum"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():

      attrs={'id': rs[1], 'name': rs[0],
             'type':'field', 'nativetype': rs[2],
             'required': rs[3] and not rs[4]}

      if rs[2] in ('int8','int2','int4','numeric',
                   'float4','float8','money','bool'):
        attrs['datatype']='number'
      elif rs[2] in ('date','time','timestamp','abstime','reltime'):
        attrs['datatype']='date'
      else:
        attrs['datatype']='text'

      if rs[5] != -1:
        attrs['length'] = rs[5]

      list.append(GDataObjects.Schema(attrs=attrs))

    cursor.close()
    return list



class PG_DataObject_Object(PG_DataObject, \
      DBSIG_DataObject_Object):

  def __init__(self):
    # Call DBSIG init first because PG_DataObject needs to overwrite
    # some of its values
    DBSIG_DataObject_Object.__init__(self)
    PG_DataObject.__init__(self)

  def _buildQuery(self, conditions={}):
    return DBSIG_DataObject_Object._buildQuery(self, conditions)


class PG_DataObject_SQL(PG_DataObject, \
      DBSIG_DataObject_SQL):
  def __init__(self):
    # Call DBSIG init first because PG_DataObject needs to overwrite
    # some of its values
    DBSIG_DataObject_SQL.__init__(self)
    PG_DataObject.__init__(self)

  def _buildQuery(self, conditions={}):
    return DBSIG_DataObject_SQL._buildQuery(self, conditions)


#
#  Extensions to Trigger Namespaces
#
class TriggerExtensions:

  def __init__(self, connection):
    self.__connection = connection

  # Return the current date, according to database
  def getTimeStamp(self):
    return self.__singleQuery("select current_timestamp")

  # Return a sequence number from sequence 'name'
  def getSequence(self, name):
    return self.__singleQuery("select nextval('%s')" % name)

  # Run the SQL statement 'statement'
  def sql(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      cursor.close()
    except:
      cursor.close()
      raise

  # Used internally
  def __singleQuery(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      rv = cursor.fetchone()
      cursor.close()
    except mesg:
      GDebug.printMesg(1,"**** Unable to execute extension query")
      GDebug.printMesg(1,"**** %s" % mesg)
      cursor.close()
      return None

    try:
      return rv[0]
    except:
      return None



######################################
#
#  The following hashes describe
#  this driver's characteristings.
#
######################################

#
#  All datasouce "types" and corresponding DataObject class
#
supportedDataObjects = {
  'object': PG_DataObject_Object,
  'sql':    PG_DataObject_SQL
}
